
(* MOToRMLFn.sml *)

functor MOToRMLFn(
			   structure Absyn : ABSYN
		       structure Control  : CONTROL
		       structure Util     : UTIL
			 ) : PARSE =
  struct

    structure Absyn = Absyn
    
    fun getTemp() = "temp"^Int.toString(Util.tick())
    
    fun bug s = Util.bug("MOToRMLFn."^s)
    
	val modelicaid = Absyn.makeIdent("MODELICARML", ref(Absyn.dummyInfo))

	fun mktuplety([ty])  = ty
		| mktuplety(tyseq) = Absyn.TUPLEty(tyseq, ref(Absyn.dummyInfo)) 
	
	fun buildClass    (ident, classdef, specs, decs) = (specs, decs)
	fun buildRecord   (ident, classdef, specs, decs) = (specs, decs)
	fun buildType     (ident, classdef, specs, decs) = (specs, decs)
	fun buildPackage  (ident, classdef, specs, decs) = (specs, decs)	
	
	fun caseType(Absyn.IDENT("Real", info))    = Absyn.IDENT("real", info)
	  | caseType(Absyn.IDENT("Integer", info)) = Absyn.IDENT("integer", info)
	  | caseType(Absyn.IDENT("String", info))  = Absyn.IDENT("string", info)
	  | caseType(x)    = x 		
	 (* only 1 DOT please *)
	fun getType(Absyn.QUALIFIED(ident1, Absyn.PATHIDENT(ident2))) = 
		Absyn.LONGID(SOME(caseType(ident1)), caseType(ident2), ref(Absyn.dummyInfo))
	| getType(Absyn.PATHIDENT(ident as Absyn.IDENT(_, info))) = 
		Absyn.LONGID(NONE, caseType(ident), info)
	| getType(_) = bug (" we only allow one DOT in name_path !")

	fun getTypes([]) = []
		| getTypes(Absyn.PUBLIC(elementItems)::rest) = 
			let fun loop ([]) = [] 
				| loop(Absyn.ELEMENTITEM(
							Absyn.ELEMENT(_,_,_,
								ident, 
								elementSpec as Absyn.COMPONENTS(_, path, components),
								info, 
								NONE))::elems) =
					let val ty =  Absyn.CONSty([], getType(path), ref(Absyn.dummyInfo))
						fun foo(x) = ty
					in List.tabulate (List.length components, foo) end
			in
				loop(elementItems) @ getTypes(rest)
			end
					
	fun sweepRecords([]) = []
		| sweepRecords(
			Absyn.ELEMENTITEM(
				Absyn.ELEMENT(_,_,_,
					ident, 
					elementSpec as Absyn.CLASSDEF(_, class as 
								Absyn.CLASS(
									identclass,
									partial,
									final,
									encapsulated,
									_,
									classdef as Absyn.PARTS(classparts, _))), 
					info, 
					NONE))::rest) = 
			let val types = getTypes(classparts)
			in
				print "sweepRecords\n";
				if (List.length classparts = 0)
				then Absyn.CONcb(identclass, 
						ref(Absyn.identCtxInfo identclass))::sweepRecords(rest)
				else Absyn.CTORcb(identclass, types, 
						ref(Absyn.identCtxInfo identclass))::sweepRecords(rest)
			end
		| sweepRecords(_) = bug(" bug in ELEMENTITEM(ELEMENT(CLASSDEF(CLASS(R_RECORD))))! ")
		   
				
	fun buildConstructors([]) = []
		| buildConstructors(Absyn.PUBLIC(elementItems)::rest) = 
			sweepRecords(elementItems) @ buildConstructors(rest)
		| buildConstructors(Absyn.PROTECTED(elementItems)::rest) = 
			sweepRecords(elementItems) @ buildConstructors(rest)
		| buildConstructors(_) = bug(" only PUBLIC and PROTECTED are handled in UNIONTYPE ")	
		
	fun buildDatatype (ident as Absyn.IDENT(name, info), classdef, specs, decs) = 
		let val classparts = case classdef of
							  Absyn.PARTS(classparts, _) => classparts
							  | _ => bug(" other classparts than PARTS are not handled in the UNIONTYPE ")
			val conbinds = buildConstructors(classparts)
		in
			print "specs\n";
			(
			specs @ [ Absyn.DATAspec([Absyn.DATBIND([], ident, conbinds, info)], [], info) ],
			decs  (* @	[ Absyn.DATAdec([Absyn.DATBIND([], ident, conbinds, info)], [], info)  ] *)
			)
		end
		
	(****************** RELATIONS ************************)	(****************** RELATIONS ************************)
	(****************** RELATIONS ************************)	(****************** RELATIONS ************************)
	(****************** RELATIONS ************************)	(****************** RELATIONS ************************)
	(*fun constructRel(Absyn.ELEMENTITEM(
				Absyn.ELEMENT(_,_,_,
					ident, 
					ementSpec as Absyn.COMPONENTS(Absyn.ATTR(_, _, direction, _), path, components),
					info, 
					NONE))::rest, ty, datbind, clause) = (ty, datbind, clause)		
	  | constructRel(Absyn.ELEMENTITEM(
				Absyn.ELEMENT(_,_,_,
					ident, 
					elementSpec as Absyn.CLASSDEF(_, class as 
								Absyn.CLASS(
									identclass,
									partial,
									final,
									encapsulated,
									_,
									classdef as Absyn.PARTS(classparts, _))), 
					info, 
					NONE))::rest, ty, datbind, clause) = (ty, datbind, clause)
	*)

  (** Modelica+ Patterns vs. RML patterns
   Pattern = MWILDpat of info ref
			| MLITpat of Exp * info ref
			| MCONpat of Path * info ref
			| MSTRUCTpat of Path option * Pattern list * info ref
			| MBINDpat of var * Pattern * info ref
			| MIDENTpat of ident * Pattern ref * info ref
    pat = WILDpat of info ref
		| LITpat of lit * info ref
		| CONpat of longid * info ref
		| STRUCTpat of longid option * pat list * info ref
		| BINDpat of var * pat * info ref
		| IDENTpat of ident * pat ref * info ref	  
    **)
	 (* only 1 DOT please *)
	fun path2Ident(Absyn.QUALIFIED(ident1, Absyn.PATHIDENT(ident2))) = 
		Absyn.LONGID(SOME(ident1), ident2, ref(Absyn.dummyInfo))
	| path2Ident(Absyn.PATHIDENT(ident as Absyn.IDENT(_, info))) = 
		Absyn.LONGID(NONE, ident, info)
	| path2Ident(_) = bug (" we only allow one DOT in name_path !")
    
    (*
    datatype lit = CCONlit of char * info ref
				 | ICONlit of int * info ref
				 | RCONlit of real * info ref
				 | SCONlit of string * info ref
	datatype Exp = INTEGER of int
	       | REAL of real
	       | STRING of string
	       | BOOL of bool
	...    
    *)
    fun makeExpLiteral(Absyn.INTEGER(icon)) = Absyn.ICONlit(icon, ref(Absyn.dummyInfo)) 
    | makeExpLiteral(Absyn.REAL(rcon)) = Absyn.RCONlit(rcon, ref(Absyn.dummyInfo)) 
    | makeExpLiteral(Absyn.STRING(scon)) = Absyn.SCONlit(scon, ref(Absyn.dummyInfo)) 
    | makeExpLiteral(_) = bug (" strange literal ") 	    
    (*
    | makeExpLiteral(Absyn.BOOL(true)) = Absyn.ICONlit(1, ref(Absyn.dummyInfo))
    | makeExpLiteral(Absyn.BOOL(false)) = Absyn.ICONlit(0, ref(Absyn.dummyInfo))
    *)
	(*
  (** - Expressions *)
  datatype Operator = ADD   | SUB    | MUL     | DIV       | POW
		    | UPLUS | UMINUS
		    | AND   | OR
		    | NOT
		    | LESS  | LESSEQ | GREATER | GREATEREQ | EQUAL | NEQUAL  

  datatype Exp = CREF of ComponentRef
	       | BOOL of bool
	       | BINARY of Exp * Operator * Exp (* Binary operations, e.g. a*b *)
	       | UNARY of Operator * Exp (* Unary operations, e.g. -(x) *)
	       | LBINARY of Exp * Operator * Exp (* Logical binary operations: and, or *)
	       | LUNARY of Operator * Exp (* Logical unary operations: not *)
	       | RELATION of Exp * Operator * Exp (* Relations, e.g. a >= 0 *)
	       | IFEXP of Exp * Exp * Exp * (Exp * Exp) list  (* If expressions *)
	       | CALL of ComponentRef * FunctionArgs (* Function calls *)
	       | ARRAY of Exp list
	       | MATRIX of Exp list list
	       | RANGE of Exp * Exp option * Exp (* Range expressions, e.g. 1:10 or 1:0.5:10 *)
	       | TUPLE of Exp list (*PR.*) (* Tuples used in function calls returning several values *)
	       | END (* array access operator for last element, e.g. a[end]:=1; *)
	       | CODE of Code (* Modelica AST Code constructors *)	

    datatype exp = LITexp of lit * info ref
				 | CONexp of longid * info ref
				 | VARexp of longid * info ref
				 | STRUCTexp of longid option * exp list * info ref
				 | IDENTexp of longid * exp ref * info ref	       
	*)	
	fun translateExpIdent(Absyn.CREF(Absyn.CREF_IDENT(ident, _))) =  
		(
		print "translateExpIdent\n";
		Absyn.VARexp(
			Absyn.LONGID(NONE, ident, ref(Absyn.dummyInfo)),
			ref(Absyn.dummyInfo))
		)
		
	
	fun translateExp(Absyn.BINARY(Exp1,Operator,Exp2)) =
		let val idop = 
			case Operator of
			Absyn.ADD      => "real_add"
			| Absyn.SUB    => "real_sub"
			| Absyn.MUL    => "real_mul"  
			| Absyn.DIV    => "real_div"    
			| Absyn.POW    => "real_pow"
		   val resstr = getTemp()
		   val result = Absyn.RETURN([Absyn.VARexp(
										Absyn.LONGID(NONE, 
											Absyn.makeIdent(resstr, ref(Absyn.dummyInfo)),
											ref(Absyn.dummyInfo)),
										ref(Absyn.dummyInfo))], ref(Absyn.dummyInfo))
		in
		 print "translateExp-Binary\n";
		 (result, 
		  Absyn.CALLgoal(
				Absyn.LONGID(NONE,
					Absyn.makeIdent(idop, ref(Absyn.dummyInfo)), 
					ref(Absyn.dummyInfo)),
				[translateExpIdent(Exp1), translateExpIdent(Exp2)],
				[Absyn.IDENTpat(Absyn.makeIdent(resstr, ref(Absyn.dummyInfo)), 
								ref(Absyn.WILDpat(ref(Absyn.dummyInfo))), 
								ref(Absyn.dummyInfo))],
				ref(Absyn.dummyInfo)))
		end
	| translateExp(Absyn.UNARY(Operator,Exp)) = 
		let val idop = 
			case Operator of
				Absyn.UMINUS => "real_neg"		
		   val resstr = getTemp()
		   val result = Absyn.RETURN([Absyn.VARexp(
										Absyn.LONGID(NONE, 
											Absyn.makeIdent(resstr, ref(Absyn.dummyInfo)),
											ref(Absyn.dummyInfo)),
										ref(Absyn.dummyInfo))], ref(Absyn.dummyInfo))
		in
		 print "translateExp-Unary\n";		 		
		 (result, 
		  Absyn.CALLgoal(
				Absyn.LONGID(NONE,
					Absyn.makeIdent(idop, ref(Absyn.dummyInfo)),
					ref(Absyn.dummyInfo)),
				[translateExpIdent(Exp)],
				[Absyn.IDENTpat(Absyn.makeIdent(resstr, ref(Absyn.dummyInfo)), 
								ref(Absyn.WILDpat(ref(Absyn.dummyInfo))), 
								ref(Absyn.dummyInfo))],
				ref(Absyn.dummyInfo)))
		end
	
	(*
	| translateExp(Absyn.LBINARY(Exp,Operator,Exp) =

	| translateExp(Absyn.LUNARY(Operator, Exp) =
		 
	| translateExp(Absyn.RELATION(Exp, Operator,Exp)
	*)
	
	
	        
    fun transformPattern(pattern) = 
		case pattern of
		  Absyn.MWILDpat(info)      => Absyn.WILDpat(info)
		| Absyn.MLITpat(Exp, info)  => Absyn.LITpat(makeExpLiteral(Exp), info)
		| Absyn.MCONpat(path, info) => Absyn.CONpat(path2Ident(path), info)
		| Absyn.MSTRUCTpat(path, patterns, info) => 
				(case path of 
					SOME(path') => Absyn.STRUCTpat(SOME(path2Ident(path')), extractPatterns(patterns), info)
					| NONE => Absyn.STRUCTpat(NONE, extractPatterns(patterns), info))
		| Absyn.MBINDpat(var, pat, info) => Absyn.BINDpat(var, transformPattern(pat), info)
		| Absyn.MIDENTpat(ident, ref(pat), info) => Absyn.IDENTpat(ident, ref(transformPattern(pat)), info)
		
	and extractPatterns(patterns) = 
		 map transformPattern patterns

	(*
    datatype goal = CALLgoal of longid * exp list * pat list * info ref
				  | EQUALgoal of var * exp * info ref
				  | LETgoal of pat * exp * info ref
				  | NOTgoal of goal * info ref
				  | ANDgoal of goal * goal * info ref
	*)
	
	fun buildGoalz(Absyn.EQUATIONITEM(equation, _), exp, decls, localdecls) = 
		(case equation of
		     Absyn.EQ_EQUALS(
				exp1 as Absyn.CREF(ComponentRef1 as Absyn.CREF_IDENT(ident1, _)),
				exp2 as Absyn.CALL(ComponentRef2 as Absyn.CREF_IDENT(ident2, _), 
				FunctionArgs as Absyn.FUNCTIONARGS(Exps, _)) ) =>
				let val goal = Absyn.CALLgoal(
					Absyn.LONGID(NONE, ident2, ref(Absyn.dummyInfo)),
					map translateExpIdent Exps,
					[Absyn.IDENTpat(
						ident1, 
						ref(Absyn.WILDpat(ref(Absyn.dummyInfo))), 
						ref(Absyn.dummyInfo))],
						ref(Absyn.dummyInfo))
				in
					goal
				end	
			| _ => bug (" we only allow equality equations "))

	fun buildGoals(x::nil, exp, decls, localdecls) =
		buildGoalz(x)			
	fun buildGoals(x::rest, exp, decls, localdecls) =
		let val (goal1) = buildGoalz(x, exp, decls, localdecls)
			val (goal2) = buildGoals(rest, exp, decls, localdecls)
		in
			Absyn.ANDgoal(goal1, goal2, ref(Absyn.dummyInfo))
		end	
			
	fun buildGoal([], exp, decls, localdecls) = 
		(NONE,
		Absyn.RETURN(
			[translateExpIdent(exp)], 
			ref(Absyn.dummyInfo))) 
		| buildGoal(seq_eq, exp, decls, localdecls) = 
			let val (result, goal2) = translateExp(exp)
			in 
			 (SOME(Absyn.ANDgoal(
						buildGoals(seq_eq, exp, decls, localdecls), 
						goal2,
						ref(Absyn.dummyInfo))), result)
			end
	(*
    datatype clause	= CLAUSE1 of goal option * ident * pat list * result * info ref
					| CLAUSE2 of clause * clause * info ref 	
	*)					
					
	fun buildClauses(ident, 
			Absyn.CASE(
				patterns,
				localdecls,
				Absyn.EQUATIONS(equations),
				exp)::nil, elems) =
		let val (goal, result) = buildGoal(equations, exp, elems, localdecls)
		in 
			Absyn.CLAUSE1(
				goal, 
				ident,
				extractPatterns(patterns),
				result, 
				ref (Absyn.dummyInfo))
		end
	| buildClauses(ident, x::rest, elems) =
		Absyn.CLAUSE2(
			buildClauses(ident, [x], elems), 
			buildClauses(ident, rest, elems), 
			ref(Absyn.identCtxInfo ident))
		
	fun constructClause(ident,
			[Absyn.ALGORITHMITEM(
				Absyn.ALG_MATCH(ComponentRef, Exp, ElementItems, Cases),
				_)], ty, datbind, clause) =
		  let val clause' = buildClauses(ident, Cases, ElementItems)::clause
		  in 
			(ty, datbind, clause')
		  end
	| constructClause(ident, _, ty, datbind, clause) = 
		    bug(" only MATCH construct is handled in algorithm sections ")


	fun constructRel(elementItems, ty, datbind, clause) = 
				([Absyn.RELty(
					[Absyn.CONSty([],
							Absyn.LONGID(NONE, Absyn.makeIdent("Exp", ref(Absyn.dummyInfo)), 
							ref(Absyn.dummyInfo)),
						ref(Absyn.dummyInfo))],
					[Absyn.CONSty([],
							Absyn.LONGID(NONE, Absyn.makeIdent("real", ref(Absyn.dummyInfo)), 
							ref(Absyn.dummyInfo)),
						ref(Absyn.dummyInfo))],
					ref(Absyn.dummyInfo))], datbind, clause) 
		  
	fun buildRelationSpecAndDec(ident, []) = []
		| buildRelationSpecAndDec(ident, part::rest) = 
		  let val ty=[]
		      val datbind=[]
		      val clause=[]
		  in
			case part of
			Absyn.PUBLIC(elementItems)       => 
				constructRel(elementItems, ty, datbind, clause)::buildRelationSpecAndDec(ident, rest)
			| Absyn.PROTECTED(elementItems)  => 
				constructRel(elementItems, ty, datbind, clause)::buildRelationSpecAndDec(ident, rest)
			| Absyn.ALGORITHMS(algorithmItems) => 
				constructClause(ident, algorithmItems, ty, datbind, clause)::buildRelationSpecAndDec(ident, rest)		
			| _ => bug(" only PUBLIC/PROTECTED/ALGORITHM are handled in FUNCTION ")	
		  end

	fun buildRelation (ident as Absyn.IDENT(name, info), classdef, specs, decs) = 
		let val classparts' = case classdef of
							  Absyn.PARTS(classparts, _) => classparts
							  | _ => bug(" other classparts than PARTS are not handled in the FUNCTION ")
			val (ty::restty, datbind::restdatbind, clause::restclause)::rest = buildRelationSpecAndDec(ident, classparts')	
		in
			print "specs\n";
			(
			specs @ [ Absyn.RELspec(ident, ty, info) ],
			decs  @	[ Absyn.RELdec([Absyn.RELBIND(ident, SOME(ty), clause, info)], info)  ]
			)
		end

	fun walkClasses(class::rest, specs, decs) = 
		let val Absyn.CLASS(ident,partial,final,encapsulated,restriction,classdef) = class
			val (s,d) = case restriction of
						  Absyn.R_CLASS  =>    ( buildClass   (ident, classdef, specs, decs) )
						| Absyn.R_MODEL  =>    ( buildClass   (ident, classdef, specs, decs) )
						| Absyn.R_RECORD =>    ( buildRecord  (ident, classdef, specs, decs) )
						| Absyn.R_TYPE   =>    ( buildType   (ident, classdef, specs, decs) )
						| Absyn.R_PACKAGE =>	 ( buildPackage (ident, classdef, specs, decs) )
						| Absyn.R_FUNCTION =>  ( buildRelation(ident, classdef, specs, decs) )
						| Absyn.R_UNIONTYPE => ( buildDatatype(ident, classdef, specs, decs) )
						| _ => bug ("RMLModelica+ can only handle now: CLASS|MODEL|RECORD|TYPE|PACKAGE|FUNCTION|UNIONTYPE!")
		in
		  print "classes\n";		
		  walkClasses(rest, s, d) 
		end		
	   | walkClasses([], specs, decs) = (specs, decs)
		

	fun buildInterfaceAndDecs(Absyn.PROGRAM(classes, within, interface, info)) =
		let val (specs, decs) = walkClasses(classes, [] (* specs *), [] (* decs *))
	        val Absyn.INTERFACE({source,...}, infoI) = interface
		in 
		  print "interface and decs\n";		
		  (Absyn.INTERFACE({modid=modelicaid, specs=specs, source=source}, infoI), decs) 
		end
		
	fun buildInfo(Absyn.PROGRAM(_, _, _, info)) = info

    fun mo2rmlmodule(Absyn.MODELICA(program)) = 
		let val (interface, decs) = buildInterfaceAndDecs(program)
		in 
			print "\nmo2rmlmodule\n";		
			Absyn.MODULE(interface, decs, buildInfo(program))
		end
		
    fun mo2rmlinterface(modelica) = 
	   let val Absyn.MODULE(Absyn.INTERFACE({modid,specs,source}, infoI), decs, infoM) = 
			   mo2rmlmodule(modelica)
	       val interface = Absyn.INTERFACE({modid=modid,specs=specs,
					       source=source}, 
					       infoI)
	   in
		 print "\nmo2rmlinterface\n";	   
	     Absyn.MODULE(interface, decs, infoM)
	   end
    
    fun parseModule    (file) = Absyn.MODULE(Absyn.dummyInterface, [], ref(Absyn.dummyInfo))
    fun parseInterface (file) = Absyn.MODULE(Absyn.dummyInterface, [], ref(Absyn.dummyInfo))
         
  end (* functor InstrumentFn *)
